#include <stdlib.h>
#include <string.h>
#include <libintl.h>
#define _(String) gettext (String)
#include <gtk/gtk.h>
#include <m17n-im-config.h>

struct ConfigSaveRevert
{
  GtkWidget *config, *save, *revert;
};

void
status_changed_cb (gboolean modified, gpointer data)
{
  struct ConfigSaveRevert *config_save_revert = data;

  if (GTK_WIDGET_SENSITIVE (config_save_revert->save))
    {
      if (! modified)
	{
	  gtk_widget_set_sensitive (config_save_revert->save, FALSE);
	  gtk_widget_set_sensitive (config_save_revert->revert, FALSE);
	}
    }
 else
   {
      if (modified)
	{
	  gtk_widget_set_sensitive (config_save_revert->save, TRUE);
	  gtk_widget_set_sensitive (config_save_revert->revert, TRUE);
	}
   }
}

static gboolean
delete (GtkWidget *widget, GdkEvent *event, gpointer data)
{
  gtk_main_quit ();
  return FALSE;
}


static void
revert_clicked_cb (GtkButton *button, gpointer data)
{
  struct ConfigSaveRevert *config_save_revert = data;

  mim_config_revert (config_save_revert->config);
  status_changed_cb (FALSE, data);
}

static void
save_clicked_cb (GtkButton *button, gpointer data)
{
  struct ConfigSaveRevert *config_save_revert = data;

  mim_config_save (config_save_revert->config);
  status_changed_cb (FALSE, data);
}

static void
quit_clicked_cb (GtkButton *button, gpointer data)
{
  struct ConfigSaveRevert *config_save_revert = data;

  if (mim_config_modified (config_save_revert->config))
    {
      GtkWidget *dialog, *label;
      gint response;

      dialog = (gtk_dialog_new_with_buttons
		("confirmation",
		 GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (button))),
		 GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
		 GTK_STOCK_NO, GTK_RESPONSE_NO,
		 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		 GTK_STOCK_SAVE, GTK_RESPONSE_YES,
		 NULL));
      label = gtk_label_new (_("Save configuration?"));
      gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), label);
      gtk_widget_show_all (dialog);
      response = gtk_dialog_run (GTK_DIALOG (dialog));
      gtk_widget_destroy (dialog);
      if (response == GTK_RESPONSE_CANCEL)
	return;
      if (response == GTK_RESPONSE_YES)
	mim_config_save (config_save_revert->config);
    }
  gtk_main_quit ();      
}


static void
ok_clicked_cb (GtkButton *button, gpointer data)
{
  struct ConfigSaveRevert *config_save_revert = data;

  if (mim_config_modified (config_save_revert->config))
    mim_config_save (config_save_revert->config);
  gtk_main_quit ();      
}

int
main (int argc, char **argv)
{
  GtkWidget *window, *vbox, *config, *hbox;
  GtkWidget *revert, *save, *ok, *quit;
  struct ConfigSaveRevert config_save_revert;
  MimConfigCallback callback;
    
#if ENABLE_NLS
  bindtextdomain ("m17n-im-config", GETTEXTDIR);
  bind_textdomain_codeset ("m17n-im-config", "UTF-8");
#endif
  gtk_init (&argc, &argv);
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_size_request (window, 500, 300);
  gtk_window_set_title (GTK_WINDOW (window), _("M17N-IM Configuration"));
  g_signal_connect (G_OBJECT (window), "destroy",
		    G_CALLBACK (gtk_main_quit), NULL);
    
  /* We create these widgets:
     +-vbox----------------------------------+
     |+-config------------------------------+|
     ||                                     ||
     ||                                     ||
     ||  M17N-IM Configuration              ||
     ||                                     ||
     ||                                     ||
     |+-------------------------------------+|
     |+-hbox--------------------------------+|
     ||          +------+ +-----++----+ +--+||
     ||          |revert| |save| |quit| |ok|||
     ||          +------+ +-----++----+ +--+||
     |+-------------------------------------+|
     +---------------------------------------+
  */

  vbox = gtk_vbox_new (FALSE, 10);
  gtk_container_add (GTK_CONTAINER (window), vbox);

  revert = gtk_button_new_from_stock (GTK_STOCK_REVERT_TO_SAVED);
  gtk_widget_set_sensitive (revert, FALSE);
  save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
  gtk_widget_set_sensitive (save, FALSE);
  quit = gtk_button_new_from_stock (GTK_STOCK_QUIT);
  ok = gtk_button_new_from_stock (GTK_STOCK_OK);

  config_save_revert.save = save;
  config_save_revert.revert = revert;
  callback.func = status_changed_cb;
  callback.arg = &config_save_revert;
  config = mim_config_widget (&callback);
  config_save_revert.config = config;
  gtk_box_pack_start (GTK_BOX (vbox), config, TRUE, TRUE, 0);

  g_signal_connect (G_OBJECT (revert), "clicked",
		    G_CALLBACK (revert_clicked_cb), &config_save_revert);
  g_signal_connect (G_OBJECT (save), "clicked",
		    G_CALLBACK (save_clicked_cb), &config_save_revert);
  g_signal_connect (G_OBJECT (quit), "clicked",
		    G_CALLBACK (quit_clicked_cb), &config_save_revert);
  g_signal_connect (G_OBJECT (ok), "clicked",
		    G_CALLBACK (ok_clicked_cb), &config_save_revert);

  hbox = gtk_hbutton_box_new ();
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_END);
  gtk_box_set_spacing (GTK_BOX (hbox), 10);
  gtk_container_add (GTK_CONTAINER (hbox), revert);
  gtk_container_add (GTK_CONTAINER (hbox), save);
  gtk_container_add (GTK_CONTAINER (hbox), quit);
  gtk_container_add (GTK_CONTAINER (hbox), ok);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

  gtk_widget_show_all (window);
  gtk_main ();

  return 0;
}
